/*                          M A I N . C
 * BRL-CAD / ADRT
 *
 * Copyright (c) 2007-2008 United States Government as represented by
 * the U.S. Army Research Laboratory.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this file; see the file named COPYING for more
 * information.
 */
/** @file main.c
 *
 */

#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#ifdef HAVE_GETOPT_H
#  include <getopt.h>
#endif

#include "bio.h"
#include "bu.h"

#include "./bench.h"


#ifdef HAVE_GETOPT_LONG
static struct option longopts[] =
{
    { "dump cache", no_argument,	NULL, 'c' },
    { "dump image",	no_argument,	NULL, 'i' },
    { "help",	no_argument,	NULL, 'h' },
};
#endif

static char shortopts[] = "cdh";


static void finish(int sig) {
    bu_exit(EXIT_FAILURE, "Collected signal %d, aborting!\n", sig);
}


static void help() {
    printf("%s\n", "usage: adrt_bench [options] [proj_env_file]\n\
  -c\t\tdump cache.\n\
  -d\t\tdump ppm image.\n\
  -h\t\tdisplay help.\n");
}


int main(int argc, char **argv) {
    int c = 0, image, cache;
    char proj[64], temp[64];


    signal(SIGINT, finish);

    if (argc == 1) {
	help();
	return EXIT_FAILURE;
    }

    /* Initialize */
    cache = 0;
    image = 0;
    proj[0] = 0;

    /* Parse command line options */

    while ((c =
#ifdef HAVE_GETOPT_LONG
	    getopt_long(argc, argv, shortopts, longopts, NULL)
#else
	    getopt(argc, argv, shortopts)
#endif
	       )!= -1)
    {
	switch (c) {
	    case 'c':
		cache = 1;
		break;
	    case 'd':
		image = 1;
		break;
	    case 'h':
		help();
		return EXIT_SUCCESS;
	    default:
		help();
		return EXIT_FAILURE;
	}
    }
    argc -= optind;
    argv += optind;

    bu_strlcpy(proj, argv[0], sizeof(proj));

    if (proj[0]) {
	bench(proj, cache, image);
    } else {
	help();
	return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

/*
 * Local Variables:
 * mode: C
 * tab-width: 8
 * indent-tabs-mode: t
 * c-file-style: "stroustrup"
 * End:
 * ex: shiftwidth=4 tabstop=8
 */
